//
//  ContentView.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import Foundation
import SwiftUI

struct ContentView: View {
    @EnvironmentObject var authManager: AuthenticationManager
    @EnvironmentObject var settingsManager: SettingsManager
    @State private var isCheckingAuth = true
    @Environment(\.scenePhase) private var scenePhase

    var body: some View {
        Group {
            if isCheckingAuth {
                AppLoadingView()
            } else if authManager.isAuthenticated {
                MainTabView()
            } else {
                LoginView()
            }
        }
        .preferredColorScheme(settingsManager.theme.colorScheme)
        .accentColor(settingsManager.accentColor.color)
        .animation(.easeInOut(duration: 0.3), value: authManager.isAuthenticated)
        .onAppear {
            checkAuthenticationStatus()
        }
    }

    private func checkAuthenticationStatus() {
        // Check authentication status first
        authManager.checkAuthenticationStatus()

        // Give it a moment to complete the check
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.5) {
            isCheckingAuth = false
        }
    }
}

struct AppLoadingView: View {
    @State private var isAnimating = false

    var body: some View {
        VStack(spacing: 20) {
            Image("AppLogo")
                .font(.system(size: 60))
                .foregroundColor(.accentColor)
                .scaleEffect(isAnimating ? 1.1 : 1.0)
                .animation(
                    .easeInOut(duration: 1.0).repeatForever(autoreverses: true), value: isAnimating)

            Text("SwiftForge")
                .font(.largeTitle)
                .fontWeight(.bold)

            ProgressView()
                .progressViewStyle(CircularProgressViewStyle(tint: .accentColor))
                .scaleEffect(1.2)
        }
        .frame(maxWidth: .infinity, maxHeight: .infinity)
        .background(Color(.systemBackground))
        .onAppear {
            isAnimating = true
        }
    }
}

struct MainTabView: View {
    @EnvironmentObject var authManager: AuthenticationManager
    @State private var selectedTab = 0

    var body: some View {
        TabView(selection: $selectedTab) {
            NavigationStack {
                RepositoriesView()
            }
            .tabItem {
                Image(systemName: "folder")
                Text("Repositories")
            }
            .tag(0)

            NavigationStack {
                ProfileView()
            }
            .tabItem {
                Image(systemName: "person.circle")
                Text("Profile")
            }
            .tag(1)
        }
        .onAppear {
        }
    }
}

struct LoginView: View {
    @EnvironmentObject var authManager: AuthenticationManager
    @State private var serverURL = ""
    @State private var token = ""
    @State private var showingTokenHelp = false

    var body: some View {
        NavigationStack {
            VStack(spacing: 24) {
                // Header
                VStack(spacing: 16) {
                    Image(systemName: "hammer.fill")
                        .font(.system(size: 60))
                        .foregroundColor(.accentColor)

                    Text("SwiftForge")
                        .font(.largeTitle)
                        .fontWeight(.bold)

                    Text("Connect to your Gitea instance")
                        .font(.subheadline)
                        .foregroundColor(.secondary)
                        .multilineTextAlignment(.center)
                }
                .padding(.top, 40)

                // Form
                VStack(spacing: 20) {
                    VStack(alignment: .leading, spacing: 8) {
                        Text("Server URL")
                            .font(.headline)

                        TextField("https://gitea.example.com", text: $serverURL)
                            .textFieldStyle(RoundedBorderTextFieldStyle())
                            .autocapitalization(.none)
                            .disableAutocorrection(true)
                            .keyboardType(.URL)
                    }

                    VStack(alignment: .leading, spacing: 8) {
                        HStack {
                            Text("Access Token")
                                .font(.headline)

                            Spacer()

                            Button("Help") {
                                showingTokenHelp = true
                            }
                            .font(.caption)
                        }

                        SecureField("Enter your access token", text: $token)
                            .textFieldStyle(RoundedBorderTextFieldStyle())
                            .autocapitalization(.none)
                            .disableAutocorrection(true)
                    }
                }
                .padding(.horizontal)

                // Error message
                if let errorMessage = authManager.errorMessage {
                    Text(errorMessage)
                        .foregroundColor(.red)
                        .font(.caption)
                        .multilineTextAlignment(.center)
                        .padding(.horizontal)
                }

                // Login button
                Button(action: login) {
                    HStack {
                        if authManager.isLoading {
                            ProgressView()
                                .progressViewStyle(CircularProgressViewStyle(tint: .white))
                                .scaleEffect(0.8)
                        } else {
                            Image(systemName: "person.badge.key")
                        }
                        Text("Sign In")
                    }
                    .frame(maxWidth: .infinity)
                    .padding()
                    .background(canLogin ? Color.accentColor : Color.gray)
                    .foregroundColor(.white)
                    .cornerRadius(10)
                }
                .disabled(!canLogin || authManager.isLoading)
                .padding(.horizontal)

                Spacer()

                // Footer
                VStack(spacing: 8) {
                    Text("Secure authentication using")
                        .font(.caption)
                        .foregroundColor(.secondary)

                    HStack(spacing: 4) {
                        Image(systemName: "lock.shield")
                            .font(.caption)
                        Text("Keychain")
                            .font(.caption)
                    }
                    .foregroundColor(.secondary)
                }
                .padding(.bottom, 20)
            }
            .navigationTitle("")
            .navigationBarHidden(true)
        }
        .sheet(isPresented: $showingTokenHelp) {
            TokenHelpView()
        }
        .onAppear {
            serverURL = authManager.serverURL
        }
    }

    private var canLogin: Bool {
        !serverURL.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty
            && !token.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty
    }

    private func login() {
        Task {
            await authManager.login(
                serverURL: serverURL.trimmingCharacters(in: .whitespacesAndNewlines),
                token: token.trimmingCharacters(in: .whitespacesAndNewlines)
            )
        }
    }
}

struct TokenHelpView: View {
    @Environment(\.dismiss) private var dismiss

    var body: some View {
        NavigationStack {
            ScrollView {
                VStack(alignment: .leading, spacing: 20) {
                    VStack(alignment: .leading, spacing: 12) {
                        Text("How to get your Access Token")
                            .font(.title2)
                            .fontWeight(.semibold)

                        Text("Follow these steps to create an access token in your Gitea instance:")
                            .foregroundColor(.secondary)
                    }

                    VStack(alignment: .leading, spacing: 16) {
                        StepView(
                            number: 1,
                            title: "Sign in to Gitea",
                            description:
                                "Open your Gitea instance in a web browser and sign in to your account."
                        )

                        StepView(
                            number: 2,
                            title: "Go to Settings",
                            description:
                                "Click on your profile picture in the top right corner and select 'Settings'."
                        )

                        StepView(
                            number: 3,
                            title: "Navigate to Applications",
                            description: "In the settings sidebar, click on 'Applications'."
                        )

                        StepView(
                            number: 4,
                            title: "Create Access Token",
                            description:
                                "Under 'Manage Access Tokens', enter a token name (e.g., 'SwiftForge') and click 'Generate Token'."
                        )

                        StepView(
                            number: 5,
                            title: "Copy the Token",
                            description:
                                "Copy the generated token immediately - you won't be able to see it again!"
                        )
                    }

                    VStack(alignment: .leading, spacing: 8) {
                        HStack {
                            Image(systemName: "exclamationmark.triangle.fill")
                                .foregroundColor(.orange)
                            Text("Important")
                                .fontWeight(.semibold)
                        }

                        Text(
                            "Keep your access token secure and don't share it with others. SwiftForge stores your token securely in the iOS Keychain."
                        )
                        .font(.caption)
                        .foregroundColor(.secondary)
                    }
                    .padding()
                    .background(Color.orange.opacity(0.1))
                    .cornerRadius(8)
                }
                .padding()
            }
            .navigationTitle("Access Token Help")
            .navigationBarTitleDisplayMode(.inline)
            .navigationBarBackButtonHidden(true)
            .toolbar {
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Done") {
                        dismiss()
                    }
                }
            }
        }
    }
}

struct StepView: View {
    let number: Int
    let title: String
    let description: String

    var body: some View {
        HStack(alignment: .top, spacing: 12) {
            Text("\(number)")
                .font(.caption)
                .fontWeight(.bold)
                .foregroundColor(.white)
                .frame(width: 24, height: 24)
                .background(Color.accentColor)
                .clipShape(Circle())

            VStack(alignment: .leading, spacing: 4) {
                Text(title)
                    .fontWeight(.medium)

                Text(description)
                    .font(.caption)
                    .foregroundColor(.secondary)
            }
        }
    }
}

#Preview {
    ContentView()
        .environmentObject(AuthenticationManager())
        .environmentObject(SettingsManager())
}
