//
//  GiteaAPIService.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import Combine
import Foundation

class GiteaAPIService: ObservableObject {
    let baseURL: String
    private let token: String
    private let session: URLSession
    private let decoder: JSONDecoder

    init(baseURL: String, token: String) {
        self.baseURL = baseURL.trimmingCharacters(in: .whitespacesAndNewlines)
        self.token = token.trimmingCharacters(in: .whitespacesAndNewlines)
        self.session = URLSession.shared
        self.decoder = JSONDecoder()

        // Configure date formatter for Gitea's ISO 8601 dates
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "yyyy-MM-dd'T'HH:mm:ssZ"
        dateFormatter.locale = Locale(identifier: "en_US_POSIX")
        dateFormatter.timeZone = TimeZone(secondsFromGMT: 0)

        decoder.dateDecodingStrategy = .formatted(dateFormatter)
    }

    // MARK: - Private Methods

    private func createRequest(endpoint: String, method: HTTPMethod = .GET, body: Data? = nil)
        -> URLRequest?
    {
        guard let url = URL(string: "\(baseURL)/api/v1\(endpoint)") else {
            return nil
        }

        var request = URLRequest(url: url)
        request.httpMethod = method.rawValue

        // Ensure token is not empty
        guard !token.isEmpty else {
            return nil
        }

        // Set Authorization header - Gitea accepts "token <token>" format
        request.setValue("token \(token)", forHTTPHeaderField: "Authorization")
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue("application/json", forHTTPHeaderField: "Accept")

        if let body = body {
            request.httpBody = body
        }

        return request
    }

    private func performRequest<T: Codable>(_ request: URLRequest, responseType: T.Type)
        async throws -> T
    {
        let (data, response) = try await session.data(for: request)

        guard let httpResponse = response as? HTTPURLResponse else {
            throw APIError.invalidResponse
        }

        guard 200...299 ~= httpResponse.statusCode else {
            let errorMessage = String(data: data, encoding: .utf8) ?? "Unknown error"

            switch httpResponse.statusCode {
            case 401:
                throw APIError.unauthorized
            case 404:
                throw APIError.notFound
            default:
                throw APIError.httpError(httpResponse.statusCode, errorMessage)
            }
        }

        do {
            return try decoder.decode(T.self, from: data)
        } catch {
            print("Decoding error: \(error)")
            if let jsonString = String(data: data, encoding: .utf8) {
                print("Response data: \(jsonString)")
            }
            throw APIError.decodingError(error)
        }
    }

    private func performRequest(_ request: URLRequest) async throws {
        let (data, response) = try await session.data(for: request)

        guard let httpResponse = response as? HTTPURLResponse else {
            throw APIError.invalidResponse
        }

        guard 200...299 ~= httpResponse.statusCode else {
            let errorMessage = String(data: data, encoding: .utf8) ?? "Unknown error"

            switch httpResponse.statusCode {
            case 401:
                throw APIError.unauthorized
            case 404:
                throw APIError.notFound
            default:
                throw APIError.httpError(httpResponse.statusCode, errorMessage)
            }
        }
    }

    // MARK: - User API

    func getCurrentUser() async throws -> User {
        guard let request = createRequest(endpoint: "/user") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: User.self)
    }

    func getUser(username: String) async throws -> User {
        guard let request = createRequest(endpoint: "/users/\(username)") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: User.self)
    }

    func getUserRepositories(username: String? = nil, page: Int = 1, limit: Int = 30) async throws
        -> [Repository]
    {
        let endpoint = username != nil ? "/users/\(username!)/repos" : "/user/repos"
        let queryParams = "?page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: endpoint + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Repository].self)
    }

    func getUserOrganizations(username: String? = nil) async throws -> [Organization] {
        let endpoint = username != nil ? "/users/\(username!)/orgs" : "/user/orgs"

        guard let request = createRequest(endpoint: endpoint) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Organization].self)
    }

    func getStarredRepositories(page: Int = 1, limit: Int = 30) async throws -> [Repository] {
        let queryParams = "?page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: "/user/starred" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Repository].self)
    }

    // MARK: - Repository API

    func getRepository(owner: String, repo: String) async throws -> Repository {
        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: Repository.self)
    }

    func getRepositoryIssues(
        owner: String, repo: String, state: IssueState? = nil, page: Int = 1, limit: Int = 30
    ) async throws -> [Issue] {
        var queryParams = "?page=\(page)&limit=\(limit)"
        if let state = state {
            queryParams += "&state=\(state.rawValue)"
        }

        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/issues" + queryParams)
        else {
            throw APIError.invalidURL
        }

        // Use manual JSON parsing to handle mixed issue/PR responses and null assignees
        let (data, _) = try await URLSession.shared.data(for: request)

        guard let jsonArray = try JSONSerialization.jsonObject(with: data) as? [[String: Any]]
        else {
            throw APIError.invalidResponse
        }

        var issues: [Issue] = []
        let decoder = JSONDecoder()
        decoder.dateDecodingStrategy = .iso8601

        for itemDict in jsonArray {
            // Skip pull requests - they have base/head fields or URL contains /pulls/
            if itemDict["base"] != nil || itemDict["head"] != nil {
                continue
            }

            if let urlString = itemDict["url"] as? String, urlString.contains("/pulls/") {
                continue
            }

            // Try to decode as Issue
            do {
                let itemData = try JSONSerialization.data(withJSONObject: itemDict)
                let issue = try decoder.decode(Issue.self, from: itemData)
                issues.append(issue)
            } catch {
                // Log but don't fail - just skip this item
                print("Failed to decode issue: \(error)")
                continue
            }
        }

        return issues
    }

    func getRepositoryPullRequests(
        owner: String, repo: String, state: IssueState? = nil, page: Int = 1, limit: Int = 30
    ) async throws -> [PullRequest] {
        var queryParams = "?page=\(page)&limit=\(limit)"
        if let state = state {
            queryParams += "&state=\(state.rawValue)"
        }

        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/pulls" + queryParams)
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [PullRequest].self)
    }

    func getRepositoryBranches(owner: String, repo: String) async throws -> [Branch] {
        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/branches") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Branch].self)
    }

    func getRepositoryCommits(
        owner: String, repo: String, sha: String? = nil, page: Int = 1, limit: Int = 30
    ) async throws -> [Commit] {
        var queryParams = "?page=\(page)&limit=\(limit)"
        if let sha = sha {
            queryParams += "&sha=\(sha)"
        }

        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/commits" + queryParams)
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Commit].self)
    }

    func getRepositoryReleases(owner: String, repo: String, page: Int = 1, limit: Int = 30)
        async throws -> [Release]
    {
        let queryParams = "?page=\(page)&limit=\(limit)"

        guard
            let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/releases" + queryParams)
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Release].self)
    }

    func createRepository(
        name: String,
        description: String? = nil,
        isPrivate: Bool = false,
        autoInit: Bool = true,
        gitignoreTemplate: String? = nil,
        licenseTemplate: String? = nil
    ) async throws -> Repository {
        let requestBody = CreateRepositoryRequest(
            name: name,
            description: description,
            private: isPrivate,
            autoInit: autoInit,
            gitignoreTemplate: gitignoreTemplate,
            licenseTemplate: licenseTemplate
        )

        let jsonData = try JSONEncoder().encode(requestBody)

        guard let request = createRequest(endpoint: "/user/repos", method: .POST, body: jsonData)
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: Repository.self)
    }

    func updateRepository(owner: String, repo: String, updateData: [String: Any]) async throws
        -> Repository
    {
        // Encode the dictionary to JSON data
        let jsonData = try JSONSerialization.data(withJSONObject: updateData)
        guard
            let request = createRequest(
                endpoint: "/repos/\(owner)/\(repo)", method: .PATCH, body: jsonData)
        else {
            throw APIError.invalidURL
        }
        return try await performRequest(request, responseType: Repository.self)
    }

    func deleteRepository(owner: String, repo: String) async throws {
        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)", method: .DELETE)
        else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    func starRepository(owner: String, repo: String) async throws {
        guard let request = createRequest(endpoint: "/user/starred/\(owner)/\(repo)", method: .PUT)
        else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    func unstarRepository(owner: String, repo: String) async throws {
        guard
            let request = createRequest(endpoint: "/user/starred/\(owner)/\(repo)", method: .DELETE)
        else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    func watchRepository(owner: String, repo: String) async throws {
        guard
            let request = createRequest(
                endpoint: "/repos/\(owner)/\(repo)/subscription", method: .PUT)
        else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    func unwatchRepository(owner: String, repo: String) async throws {
        guard
            let request = createRequest(
                endpoint: "/repos/\(owner)/\(repo)/subscription", method: .DELETE)
        else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    // MARK: - Issue API

    func getIssue(owner: String, repo: String, index: Int) async throws -> Issue {
        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/issues/\(index)")
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: Issue.self)
    }

    func getIssueComments(owner: String, repo: String, index: Int) async throws -> [Comment] {
        guard
            let request = createRequest(
                endpoint: "/repos/\(owner)/\(repo)/issues/\(index)/comments")
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Comment].self)
    }

    func createIssueComment(owner: String, repo: String, index: Int, body: String) async throws
        -> Comment
    {
        let commentData = ["body": body]
        let jsonData = try JSONSerialization.data(withJSONObject: commentData)

        guard
            let request = createRequest(
                endpoint: "/repos/\(owner)/\(repo)/issues/\(index)/comments", method: .POST,
                body: jsonData)
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: Comment.self)
    }

    // MARK: - Pull Request API

    func getPullRequest(owner: String, repo: String, index: Int) async throws -> PullRequest {
        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/pulls/\(index)") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: PullRequest.self)
    }

    func getPullRequestFiles(owner: String, repo: String, index: Int) async throws -> [CommitFile] {
        guard let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/pulls/\(index)/files")
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [CommitFile].self)
    }

    func getPullRequestCommits(owner: String, repo: String, index: Int) async throws -> [Commit] {
        guard
            let request = createRequest(endpoint: "/repos/\(owner)/\(repo)/pulls/\(index)/commits")
        else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Commit].self)
    }

    // MARK: - Notifications API

    func getNotifications(all: Bool = false, page: Int = 1, limit: Int = 30) async throws
        -> [Notification]
    {
        let queryParams = "?page=\(page)&limit=\(limit)&all=\(all)"

        guard let request = createRequest(endpoint: "/notifications" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Notification].self)
    }

    func markNotificationAsRead(id: Int) async throws {
        guard let request = createRequest(endpoint: "/notifications/threads/\(id)", method: .PATCH)
        else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    func markAllNotificationsAsRead() async throws {
        guard let request = createRequest(endpoint: "/notifications", method: .PUT) else {
            throw APIError.invalidURL
        }

        try await performRequest(request)
    }

    // MARK: - Search API

    func searchRepositories(query: String, page: Int = 1, limit: Int = 30) async throws
        -> SearchResults<Repository>
    {
        let encodedQuery =
            query.addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) ?? ""
        let queryParams = "?q=\(encodedQuery)&page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: "/repos/search" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: SearchResults<Repository>.self)
    }

    func searchIssues(query: String, page: Int = 1, limit: Int = 30) async throws -> SearchResults<
        Issue
    > {
        let encodedQuery =
            query.addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) ?? ""
        let queryParams = "?q=\(encodedQuery)&page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: "/repos/issues/search" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: SearchResults<Issue>.self)
    }

    func searchUsers(query: String, page: Int = 1, limit: Int = 30) async throws -> SearchResults<
        User
    > {
        let encodedQuery =
            query.addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) ?? ""
        let queryParams = "?q=\(encodedQuery)&page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: "/users/search" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: SearchResults<User>.self)
    }

    // MARK: - Organization API

    func getOrganization(org: String) async throws -> Organization {
        guard let request = createRequest(endpoint: "/orgs/\(org)") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: Organization.self)
    }

    func getOrganizationRepositories(org: String, page: Int = 1, limit: Int = 30) async throws
        -> [Repository]
    {
        let queryParams = "?page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: "/orgs/\(org)/repos" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [Repository].self)
    }

    func getOrganizationMembers(org: String, page: Int = 1, limit: Int = 30) async throws -> [User]
    {
        let queryParams = "?page=\(page)&limit=\(limit)"

        guard let request = createRequest(endpoint: "/orgs/\(org)/members" + queryParams) else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: [User].self)
    }

    // MARK: - Server Info API

    func getServerVersion() async throws -> ServerVersion {
        guard let request = createRequest(endpoint: "/version") else {
            throw APIError.invalidURL
        }

        return try await performRequest(request, responseType: ServerVersion.self)
    }

    // MARK: - User Settings API

    func updateUserSettings(settings: UserSettingsOptions) async throws -> User {
        print("DEBUG: updateUserSettings called with settings: \(settings)")

        let encoder = JSONEncoder()
        encoder.keyEncodingStrategy = .convertToSnakeCase

        guard let settingsData = try? encoder.encode(settings) else {
            print("DEBUG: Failed to encode settings")
            throw APIError.invalidRequest
        }

        if let jsonString = String(data: settingsData, encoding: .utf8) {
            print("DEBUG: Request JSON: \(jsonString)")
        }

        // Try the user settings endpoint first (most likely to work)
        guard
            let request = createRequest(
                endpoint: "/user/settings", method: .PATCH, body: settingsData)
        else {
            print("DEBUG: Failed to create request for /user/settings")
            throw APIError.invalidRequest
        }

        print("DEBUG: Request headers: \(request.allHTTPHeaderFields ?? [:])")

        do {
            // Try to update settings - this returns UserSettingsResponse, not User
            let _ = try await performRequest(request, responseType: UserSettingsResponse.self)
            print("DEBUG: updateUserSettings succeeded with PATCH /user/settings")

            // Now fetch the updated user data
            print("DEBUG: Fetching updated user data")
            let updatedUser = try await getCurrentUser()
            print("DEBUG: Successfully fetched updated user data")
            return updatedUser

        } catch let error as APIError {
            print("DEBUG: PATCH to /user/settings failed: \(error)")

            // If settings update failed, try the user endpoint as fallback
            guard
                let userRequest = createRequest(
                    endpoint: "/user", method: .PATCH, body: settingsData)
            else {
                print("DEBUG: Failed to create request for /user")
                throw error
            }

            do {
                let result = try await performRequest(userRequest, responseType: User.self)
                print("DEBUG: updateUserSettings succeeded with PATCH /user")
                return result
            } catch {
                print("DEBUG: PATCH to /user also failed: \(error)")
                throw error
            }
        } catch {
            print("DEBUG: updateUserSettings failed with unexpected error: \(error)")
            throw error
        }
    }
}

// MARK: - HTTP Method

enum HTTPMethod: String {
    case GET = "GET"
    case POST = "POST"
    case PUT = "PUT"
    case DELETE = "DELETE"
    case PATCH = "PATCH"
}

// MARK: - API Errors

// MARK: - Request Models

struct CreateRepositoryRequest: Codable {
    let name: String
    let description: String?
    let `private`: Bool
    let autoInit: Bool
    let gitignoreTemplate: String?
    let licenseTemplate: String?

    enum CodingKeys: String, CodingKey {
        case name, description
        case `private` = "private"
        case autoInit = "auto_init"
        case gitignoreTemplate = "gitignore_template"
        case licenseTemplate = "license_template"
    }
}

struct UserSettingsOptions: Codable {
    let fullName: String?
    let email: String?
    let description: String?
    let website: String?
    let location: String?
    let hideEmail: Bool?
    let hideActivity: Bool?
    let language: String?
    let theme: String?
    let diffViewStyle: String?

    enum CodingKeys: String, CodingKey {
        case fullName = "full_name"
        case email = "email"
        case description = "description"
        case website = "website"
        case location = "location"
        case hideEmail = "hide_email"
        case hideActivity = "hide_activity"
        case language = "language"
        case theme = "theme"
        case diffViewStyle = "diff_view_style"
    }
}

// UserSettingsResponse has the same structure as UserSettingsOptions for the API response
typealias UserSettingsResponse = UserSettingsOptions

// MARK: - API Errors

enum APIError: LocalizedError {
    case invalidURL
    case invalidResponse
    case invalidRequest
    case unauthorized
    case notFound
    case httpError(Int, String)
    case decodingError(Error)
    case networkError(Error)

    var errorDescription: String? {
        switch self {
        case .invalidURL:
            return "Invalid URL"
        case .invalidResponse:
            return "Invalid response"
        case .invalidRequest:
            return "Invalid request"
        case .unauthorized:
            return "Unauthorized - Invalid access token"
        case .notFound:
            return "Server not found"
        case .httpError(let code, let message):
            return "HTTP Error \(code): \(message)"
        case .decodingError(let error):
            return "Decoding error: \(error.localizedDescription)"
        case .networkError(let error):
            return "Network error: \(error.localizedDescription)"
        }
    }
}
