//
//  SettingsManager.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import Foundation
import SwiftUI

@MainActor
class SettingsManager: ObservableObject {
    // MARK: - Published Properties

    @Published var theme: AppTheme = .system {
        didSet {
            UserDefaults.standard.set(theme.rawValue, forKey: "app_theme")
        }
    }

    @Published var accentColor: AppAccentColor = .blue {
        didSet {
            UserDefaults.standard.set(accentColor.rawValue, forKey: "accent_color")
        }
    }

    @Published var repositoryListStyle: RepositoryListStyle = .detailed {
        didSet {
            UserDefaults.standard.set(repositoryListStyle.rawValue, forKey: "repository_list_style")
        }
    }

    @Published var defaultRepositorySort: RepositorySort = .updated {
        didSet {
            UserDefaults.standard.set(defaultRepositorySort.rawValue, forKey: "default_repository_sort")
        }
    }

    @Published var showPrivateRepositories: Bool = true {
        didSet {
            UserDefaults.standard.set(showPrivateRepositories, forKey: "show_private_repositories")
        }
    }

    @Published var showForkedRepositories: Bool = true {
        didSet {
            UserDefaults.standard.set(showForkedRepositories, forKey: "show_forked_repositories")
        }
    }

    @Published var enableNotifications: Bool = true {
        didSet {
            UserDefaults.standard.set(enableNotifications, forKey: "enable_notifications")
        }
    }

    @Published var notificationTypes: Set<NotificationType> = [.issue, .pullRequest] {
        didSet {
            let typeStrings = notificationTypes.map { $0.rawValue }
            UserDefaults.standard.set(typeStrings, forKey: "notification_types")
        }
    }

    @Published var enablePushNotifications: Bool = false {
        didSet {
            UserDefaults.standard.set(enablePushNotifications, forKey: "enable_push_notifications")
        }
    }

    @Published var notificationSoundEnabled: Bool = true {
        didSet {
            UserDefaults.standard.set(notificationSoundEnabled, forKey: "notification_sound_enabled")
        }
    }

    @Published var cacheImages: Bool = true {
        didSet {
            UserDefaults.standard.set(cacheImages, forKey: "cache_images")
        }
    }

    @Published var autoRefreshInterval: RefreshInterval = .minutes5 {
        didSet {
            UserDefaults.standard.set(autoRefreshInterval.rawValue, forKey: "auto_refresh_interval")
        }
    }

    @Published var enableAutoRefresh: Bool = true {
        didSet {
            UserDefaults.standard.set(enableAutoRefresh, forKey: "enable_auto_refresh")
        }
    }

    @Published var defaultIssueFilter: IssueState = .open {
        didSet {
            UserDefaults.standard.set(defaultIssueFilter.rawValue, forKey: "default_issue_filter")
        }
    }

    @Published var defaultPRFilter: IssueState = .open {
        didSet {
            UserDefaults.standard.set(defaultPRFilter.rawValue, forKey: "default_pr_filter")
        }
    }

    @Published var enableBiometricAuth: Bool = false {
        didSet {
            UserDefaults.standard.set(enableBiometricAuth, forKey: "enable_biometric_auth")
        }
    }

    @Published var requireAuthOnLaunch: Bool = false {
        didSet {
            UserDefaults.standard.set(requireAuthOnLaunch, forKey: "require_auth_on_launch")
        }
    }

    @Published var enableAnalytics: Bool = false {
        didSet {
            UserDefaults.standard.set(enableAnalytics, forKey: "enable_analytics")
        }
    }

    @Published var enableCrashReporting: Bool = true {
        didSet {
            UserDefaults.standard.set(enableCrashReporting, forKey: "enable_crash_reporting")
        }
    }

    // MARK: - Initialization

    init() {
        loadSettings()
    }

    // MARK: - Public Methods

    func loadSettings() {
        // Theme
        if let themeString = UserDefaults.standard.string(forKey: "app_theme"),
           let savedTheme = AppTheme(rawValue: themeString) {
            theme = savedTheme
        }

        // Accent Color
        if let colorString = UserDefaults.standard.string(forKey: "accent_color"),
           let savedColor = AppAccentColor(rawValue: colorString) {
            accentColor = savedColor
        }

        // Repository List Style
        if let styleString = UserDefaults.standard.string(forKey: "repository_list_style"),
           let savedStyle = RepositoryListStyle(rawValue: styleString) {
            repositoryListStyle = savedStyle
        }

        // Repository Sort
        if let sortString = UserDefaults.standard.string(forKey: "default_repository_sort"),
           let savedSort = RepositorySort(rawValue: sortString) {
            defaultRepositorySort = savedSort
        }

        // Boolean settings
        showPrivateRepositories = UserDefaults.standard.bool(forKey: "show_private_repositories")
        showForkedRepositories = UserDefaults.standard.bool(forKey: "show_forked_repositories")
        enableNotifications = UserDefaults.standard.bool(forKey: "enable_notifications")
        enablePushNotifications = UserDefaults.standard.bool(forKey: "enable_push_notifications")
        notificationSoundEnabled = UserDefaults.standard.bool(forKey: "notification_sound_enabled")
        cacheImages = UserDefaults.standard.bool(forKey: "cache_images")
        enableAutoRefresh = UserDefaults.standard.bool(forKey: "enable_auto_refresh")
        enableBiometricAuth = UserDefaults.standard.bool(forKey: "enable_biometric_auth")
        requireAuthOnLaunch = UserDefaults.standard.bool(forKey: "require_auth_on_launch")
        enableAnalytics = UserDefaults.standard.bool(forKey: "enable_analytics")
        enableCrashReporting = UserDefaults.standard.bool(forKey: "enable_crash_reporting")

        // Notification Types
        if let typeStrings = UserDefaults.standard.array(forKey: "notification_types") as? [String] {
            notificationTypes = Set(typeStrings.compactMap { NotificationType(rawValue: $0) })
        }

        // Refresh Interval
        if let intervalString = UserDefaults.standard.string(forKey: "auto_refresh_interval"),
           let savedInterval = RefreshInterval(rawValue: intervalString) {
            autoRefreshInterval = savedInterval
        }

        // Default Filters
        if let issueFilterString = UserDefaults.standard.string(forKey: "default_issue_filter"),
           let savedFilter = IssueState(rawValue: issueFilterString) {
            defaultIssueFilter = savedFilter
        }

        if let prFilterString = UserDefaults.standard.string(forKey: "default_pr_filter"),
           let savedFilter = IssueState(rawValue: prFilterString) {
            defaultPRFilter = savedFilter
        }

        // Set default values for new installations
        if !UserDefaults.standard.bool(forKey: "settings_initialized") {
            setDefaultValues()
            UserDefaults.standard.set(true, forKey: "settings_initialized")
        }
    }

    func resetToDefaults() {
        theme = .system
        accentColor = .blue
        repositoryListStyle = .detailed
        defaultRepositorySort = .updated
        showPrivateRepositories = true
        showForkedRepositories = true
        enableNotifications = true
        notificationTypes = [.issue, .pullRequest]
        enablePushNotifications = false
        notificationSoundEnabled = true
        cacheImages = true
        autoRefreshInterval = .minutes5
        enableAutoRefresh = true
        defaultIssueFilter = .open
        defaultPRFilter = .open
        enableBiometricAuth = false
        requireAuthOnLaunch = false
        enableAnalytics = false
        enableCrashReporting = true
    }

    func clearAllSettings() {
        let keys = [
            "app_theme", "accent_color", "repository_list_style", "default_repository_sort",
            "show_private_repositories", "show_forked_repositories", "enable_notifications",
            "notification_types", "enable_push_notifications", "notification_sound_enabled",
            "cache_images", "auto_refresh_interval", "enable_auto_refresh",
            "default_issue_filter", "default_pr_filter", "enable_biometric_auth",
            "require_auth_on_launch", "enable_analytics", "enable_crash_reporting",
            "settings_initialized"
        ]

        keys.forEach { UserDefaults.standard.removeObject(forKey: $0) }
        loadSettings()
    }

    // MARK: - Private Methods

    private func setDefaultValues() {
        showPrivateRepositories = true
        showForkedRepositories = true
        enableNotifications = true
        notificationSoundEnabled = true
        cacheImages = true
        enableAutoRefresh = true
        enableCrashReporting = true
    }
}

// MARK: - Enums

enum AppTheme: String, CaseIterable, Identifiable {
    case light = "light"
    case dark = "dark"
    case system = "system"

    var id: String { rawValue }

    var displayName: String {
        switch self {
        case .light: return "Light"
        case .dark: return "Dark"
        case .system: return "System"
        }
    }

    var colorScheme: ColorScheme? {
        switch self {
        case .light: return .light
        case .dark: return .dark
        case .system: return nil
        }
    }
}

enum AppAccentColor: String, CaseIterable, Identifiable {
    case blue = "blue"
    case green = "green"
    case orange = "orange"
    case pink = "pink"
    case purple = "purple"
    case red = "red"
    case teal = "teal"
    case yellow = "yellow"

    var id: String { rawValue }

    var displayName: String {
        switch self {
        case .blue: return "Blue"
        case .green: return "Green"
        case .orange: return "Orange"
        case .pink: return "Pink"
        case .purple: return "Purple"
        case .red: return "Red"
        case .teal: return "Teal"
        case .yellow: return "Yellow"
        }
    }

    var color: Color {
        switch self {
        case .blue: return .blue
        case .green: return .green
        case .orange: return .orange
        case .pink: return .pink
        case .purple: return .purple
        case .red: return .red
        case .teal: return .teal
        case .yellow: return .yellow
        }
    }
}

enum RepositoryListStyle: String, CaseIterable, Identifiable {
    case compact = "compact"
    case detailed = "detailed"

    var id: String { rawValue }

    var displayName: String {
        switch self {
        case .compact: return "Compact"
        case .detailed: return "Detailed"
        }
    }
}

enum RepositorySort: String, CaseIterable, Identifiable {
    case name = "name"
    case updated = "updated"
    case created = "created"
    case stars = "stars"
    case forks = "forks"

    var id: String { rawValue }

    var displayName: String {
        switch self {
        case .name: return "Name"
        case .updated: return "Last Updated"
        case .created: return "Created"
        case .stars: return "Stars"
        case .forks: return "Forks"
        }
    }
}

enum RefreshInterval: String, CaseIterable, Identifiable {
    case minutes1 = "1min"
    case minutes5 = "5min"
    case minutes15 = "15min"
    case minutes30 = "30min"
    case hour1 = "1hour"
    case never = "never"

    var id: String { rawValue }

    var displayName: String {
        switch self {
        case .minutes1: return "1 minute"
        case .minutes5: return "5 minutes"
        case .minutes15: return "15 minutes"
        case .minutes30: return "30 minutes"
        case .hour1: return "1 hour"
        case .never: return "Never"
        }
    }

    var timeInterval: TimeInterval? {
        switch self {
        case .minutes1: return 60
        case .minutes5: return 300
        case .minutes15: return 900
        case .minutes30: return 1800
        case .hour1: return 3600
        case .never: return nil
        }
    }
}
