//
//  SharedViews.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import Foundation
import SwiftUI

// MARK: - Common Views

struct CustomHeader: View {
    let title: String
    let showBackButton: Bool
    let backAction: (() -> Void)?
    let trailingContent: (() -> AnyView)?

    init(
        title: String,
        showBackButton: Bool = false,
        backAction: (() -> Void)? = nil,
        @ViewBuilder trailingContent: @escaping () -> AnyView = { AnyView(EmptyView()) }
    ) {
        self.title = title
        self.showBackButton = showBackButton
        self.backAction = backAction
        self.trailingContent = trailingContent
    }

    var body: some View {
        VStack(spacing: 0) {
            HStack {
                if showBackButton {
                    Button(action: backAction ?? {}) {
                        Image(systemName: "chevron.left")
                            .font(.system(size: 18, weight: .semibold))
                            .foregroundColor(.primary)
                    }
                }

                Text(title)
                    .font(.largeTitle)
                    .fontWeight(.bold)
                    .foregroundColor(.primary)

                Spacer()

                trailingContent?()
            }
            .padding(.horizontal, 16)
            .padding(.top, 8)
            .padding(.bottom, 12)
            .background(Color.clear)

            // Subtle separator
            Rectangle()
                .fill(Color.primary.opacity(0.1))
                .frame(height: 0.5)
        }
        .background(Color.clear)
    }
}

struct ErrorView: View {
    let message: String
    let retryAction: () -> Void

    var body: some View {
        VStack(spacing: 16) {
            Image(systemName: "exclamationmark.triangle")
                .font(.system(size: 50))
                .foregroundColor(.orange)

            Text("Something went wrong")
                .font(.headline)

            Text(message)
                .font(.subheadline)
                .foregroundColor(.secondary)
                .multilineTextAlignment(.center)

            Button("Try Again", action: retryAction)
                .buttonStyle(.borderedProminent)
        }
        .padding()
    }
}

struct LoadingView: View {
    let message: String

    var body: some View {
        VStack(spacing: 16) {
            ProgressView()
                .progressViewStyle(CircularProgressViewStyle())
                .scaleEffect(1.2)

            Text(message)
                .font(.subheadline)
                .foregroundColor(.secondary)
        }
        .frame(maxWidth: .infinity, maxHeight: .infinity)
    }
}

struct EmptyStateView: View {
    let systemImage: String
    let title: String
    let description: String
    let actionTitle: String?
    let action: (() -> Void)?

    init(
        systemImage: String, title: String, description: String, actionTitle: String? = nil,
        action: (() -> Void)? = nil
    ) {
        self.systemImage = systemImage
        self.title = title
        self.description = description
        self.actionTitle = actionTitle
        self.action = action
    }

    var body: some View {
        VStack(spacing: 20) {
            Image(systemName: systemImage)
                .font(.system(size: 60))
                .foregroundColor(.secondary)

            Text(title)
                .font(.title2)
                .fontWeight(.semibold)

            Text(description)
                .font(.subheadline)
                .foregroundColor(.secondary)
                .multilineTextAlignment(.center)

            if let actionTitle = actionTitle, let action = action {
                Button(actionTitle, action: action)
                    .buttonStyle(.borderedProminent)
            }
        }
        .padding()
    }
}

// MARK: - Extensions

extension Date {
    func timeAgoSince() -> String {
        let formatter = RelativeDateTimeFormatter()
        formatter.unitsStyle = .abbreviated
        return formatter.localizedString(for: self, relativeTo: Date())
    }

    func formattedDate() -> String {
        let formatter = DateFormatter()
        formatter.dateStyle = .medium
        formatter.timeStyle = .none
        return formatter.string(from: self)
    }

    func formattedDateTime() -> String {
        let formatter = DateFormatter()
        formatter.dateStyle = .medium
        formatter.timeStyle = .short
        return formatter.string(from: self)
    }
}

extension Color {
    init?(hex: String) {
        let hex = hex.trimmingCharacters(in: CharacterSet.alphanumerics.inverted)
        var int: UInt64 = 0
        Scanner(string: hex).scanHexInt64(&int)
        let a: UInt64
        let r: UInt64
        let g: UInt64
        let b: UInt64
        switch hex.count {
        case 3:  // RGB (12-bit)
            (a, r, g, b) = (255, (int >> 8) * 17, (int >> 4 & 0xF) * 17, (int & 0xF) * 17)
        case 6:  // RGB (24-bit)
            (a, r, g, b) = (255, int >> 16, int >> 8 & 0xFF, int & 0xFF)
        case 8:  // ARGB (32-bit)
            (a, r, g, b) = (int >> 24, int >> 16 & 0xFF, int >> 8 & 0xFF, int & 0xFF)
        default:
            return nil
        }

        self.init(
            .sRGB,
            red: Double(r) / 255,
            green: Double(g) / 255,
            blue: Double(b) / 255,
            opacity: Double(a) / 255
        )
    }
}

extension String {
    func truncated(to length: Int) -> String {
        if self.count > length {
            return String(self.prefix(length)) + "..."
        } else {
            return self
        }
    }
}

// MARK: - Language Colors

func colorForLanguage(_ language: String) -> Color {
    switch language.lowercased() {
    case "swift": return .orange
    case "objective-c": return .blue
    case "javascript": return .yellow
    case "typescript": return .blue
    case "python": return .green
    case "java": return .red
    case "kotlin": return .purple
    case "rust": return .orange
    case "go": return .cyan
    case "c++", "cpp": return .blue
    case "c": return .gray
    case "c#": return .purple
    case "ruby": return .red
    case "php": return .purple
    case "html": return .orange
    case "css": return .blue
    case "vue": return .green
    case "react": return .cyan
    case "dart": return .blue
    case "flutter": return .blue
    case "shell": return .green
    case "bash": return .green
    case "dockerfile": return .cyan
    case "yaml", "yml": return .purple
    case "json": return .yellow
    case "xml": return .orange
    case "sql": return .blue
    case "r": return .blue
    case "scala": return .red
    case "haskell": return .purple
    case "clojure": return .green
    case "elixir": return .purple
    case "erlang": return .red
    case "lua": return .blue
    case "perl": return .blue
    case "powershell": return .blue
    case "vim script": return .green
    default: return .gray
    }
}

// MARK: - View Modifiers

struct CardStyle: ViewModifier {
    func body(content: Content) -> some View {
        content
            .cornerRadius(8)
            .shadow(color: Color.black.opacity(0.1), radius: 1, x: 0, y: 1)
    }
}

struct ListRowStyle: ViewModifier {
    func body(content: Content) -> some View {
        content
            .listRowInsets(EdgeInsets(top: 8, leading: 16, bottom: 8, trailing: 16))
            .listRowSeparator(.hidden)
    }
}

extension View {
    func cardStyle() -> some View {
        modifier(CardStyle())
    }

    func listRowStyle() -> some View {
        modifier(ListRowStyle())
    }

    func customHeader(
        _ title: String, showBackButton: Bool = false, backAction: (() -> Void)? = nil,
        @ViewBuilder trailingContent: @escaping () -> AnyView = { AnyView(EmptyView()) }
    ) -> some View {
        VStack(spacing: 0) {
            CustomHeader(
                title: title,
                showBackButton: showBackButton,
                backAction: backAction,
                trailingContent: trailingContent
            )
            self
        }
    }
}

struct GlassEffectModifier<S: Shape>: ViewModifier {
    let shape: S
    let namespace: Namespace.ID?

    init(shape: S, namespace: Namespace.ID? = nil) {
        self.shape = shape
        self.namespace = namespace
    }

    func body(content: Content) -> some View {
        if #available(iOS 26.0, macOS 26.0, *) {
            GlassEffectContainer {
                if let ns = namespace {
                    content
                        .glassEffect(in: shape)
                        .glassEffectID("glass", in: ns)
                } else {
                    content
                        .glassEffect(in: shape)
                }
            }
        } else {
            content
                .background(
                    shape
                        .fill(.regularMaterial)
                        .overlay(
                            shape
                                .stroke(Color.primary.opacity(0.1), lineWidth: 0.5)
                        )
                )
                .clipShape(shape)
        }
    }
}

// MARK: - Common Constants

struct AppConstants {
    static let minimumRefreshInterval: TimeInterval = 30
    static let defaultPageSize = 30
    static let maxDescriptionLength = 100
    static let maxTitleLength = 50

    struct Animation {
        static let defaultDuration: Double = 0.3
        static let fastDuration: Double = 0.15
        static let slowDuration: Double = 0.5
    }

    struct Spacing {
        static let small: CGFloat = 8
        static let medium: CGFloat = 16
        static let large: CGFloat = 24
        static let extraLarge: CGFloat = 32
    }

    struct CornerRadius {
        static let small: CGFloat = 4
        static let medium: CGFloat = 8
        static let large: CGFloat = 12
        static let extraLarge: CGFloat = 16
    }
}

// MARK: - Preview Helpers

#if DEBUG
    struct PreviewData {
        static let sampleUser = User(
            id: 1,
            login: "johndoe",
            fullName: "John Doe",
            email: "john@example.com",
            avatarUrl: "https://xsgames.co/randomusers/avatar.php?g=pixel",
            htmlUrl: "https://github.com/johndoe",
            description: "Software Developer",
            website: "https://johndoe.com",
            location: "San Francisco, CA",
            active: true,
            isAdmin: false,
            followersCount: 1337,
            followingCount: 420,
            starredReposCount: 69,
            created: Date(),
            lastLogin: Date()
        )

        static let sampleRepository = Repository(
            id: 1,
            name: "sample-repo",
            fullName: "johndoe/sample-repo",
            description: "A sample repository for testing",
            htmlUrl: "https://github.com/johndoe/sample-repo",
            cloneUrl: "https://github.com/johndoe/sample-repo.git",
            sshUrl: "git@github.com:johndoe/sample-repo.git",
            owner: sampleUser,
            private: false,
            fork: false,
            template: false,
            empty: false,
            archived: false,
            mirror: false,
            size: 1024,
            language: "Swift",
            languagesUrl: "https://api.github.com/repos/johndoe/sample-repo/languages",
            forksCount: 10,
            stargazersCount: 50,
            watchersCount: 25,
            openIssuesCount: 5,
            openPrCounter: 2,
            releaseCounter: 3,
            defaultBranch: "main",
            createdAt: Date(),
            updatedAt: Date(),
            permissions: nil,
            hasIssues: true,
            hasWiki: true,
            hasPullRequests: true,
            hasProjects: true,
            hasReleases: true,
            hasPackages: false,
            hasActions: true,
            topics: ["ios", "swift", "mobile"],
            avatarUrl: nil,
            internalTracker: nil,
            externalTracker: nil,
            externalWiki: nil
        )
    }
#endif

