//
//  EditProfileView.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-01-14.
//

import SwiftUI

struct EditProfileView: View {
    @Environment(\.dismiss) private var dismiss
    @EnvironmentObject var authManager: AuthenticationManager

    @State private var fullName: String = ""
    @State private var email: String = ""
    @State private var description: String = ""
    @State private var website: String = ""
    @State private var location: String = ""

    @State private var isLoading = false
    @State private var errorMessage: String?
    @State private var showingError = false

    // Store original values to track changes
    @State private var originalFullName: String = ""
    @State private var originalEmail: String = ""
    @State private var originalDescription: String = ""
    @State private var originalWebsite: String = ""
    @State private var originalLocation: String = ""

    var body: some View {
        NavigationView {
            Form {
                Section(header: Text("Basic Information")) {
                    HStack {
                        Text("Full Name")
                        Spacer()
                        TextField("Enter your full name", text: $fullName)
                            .textFieldStyle(RoundedBorderTextFieldStyle())
                            .frame(maxWidth: 200)
                    }

                    HStack {
                        Text("Email")
                        Spacer()
                        TextField("Enter your email", text: $email)
                            .textFieldStyle(RoundedBorderTextFieldStyle())
                            .keyboardType(.emailAddress)
                            .textInputAutocapitalization(.never)
                            .frame(maxWidth: 200)
                    }
                }

                Section(header: Text("About")) {
                    VStack(alignment: .leading, spacing: 8) {
                        Text("Description")
                            .font(.subheadline)
                            .foregroundColor(.secondary)

                        TextEditor(text: $description)
                            .frame(minHeight: 80)
                            .overlay(
                                RoundedRectangle(cornerRadius: 8)
                                    .stroke(Color.gray.opacity(0.3), lineWidth: 1)
                            )
                    }

                    HStack {
                        Text("Website")
                        Spacer()
                        TextField("https://example.com", text: $website)
                            .textFieldStyle(RoundedBorderTextFieldStyle())
                            .keyboardType(.URL)
                            .textInputAutocapitalization(.never)
                            .frame(maxWidth: 200)
                    }

                    HStack {
                        Text("Location")
                        Spacer()
                        TextField("Enter your location", text: $location)
                            .textFieldStyle(RoundedBorderTextFieldStyle())
                            .frame(maxWidth: 200)
                    }
                }

                if let errorMessage = errorMessage {
                    Section {
                        Text(errorMessage)
                            .foregroundColor(.red)
                            .font(.caption)
                    }
                }
            }
            .navigationTitle("Edit Profile")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button("Cancel") {
                        dismiss()
                    }
                    .disabled(isLoading)
                }

                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Save") {
                        Task {
                            await saveProfile()
                        }
                    }
                    .disabled(isLoading || !isFormValid || !hasChanges)
                }
            }
            .onAppear {
                loadCurrentUserData()
            }
            .navigationBarTitleDisplayMode(.inline)
            .toolbarBackground(.visible, for: .navigationBar)
            .overlay {
                if isLoading {
                    ZStack {
                        Color.black.opacity(0.1)
                            .ignoresSafeArea()

                        ProgressView("Saving...")
                            .padding()
                            .background(Color(.systemBackground))
                            .cornerRadius(10)
                            .shadow(radius: 10)
                    }
                }
            }
        }
    }

    private var hasChanges: Bool {
        let trimmedFullName = fullName.trimmingCharacters(in: .whitespacesAndNewlines)
        let trimmedEmail = email.trimmingCharacters(in: .whitespacesAndNewlines)
        let trimmedDescription = description.trimmingCharacters(in: .whitespacesAndNewlines)
        let trimmedWebsite = website.trimmingCharacters(in: .whitespacesAndNewlines)
        let trimmedLocation = location.trimmingCharacters(in: .whitespacesAndNewlines)

        return trimmedFullName != originalFullName || trimmedEmail != originalEmail
            || trimmedDescription != originalDescription || trimmedWebsite != originalWebsite
            || trimmedLocation != originalLocation
    }

    private var isFormValid: Bool {
        let trimmedEmail = email.trimmingCharacters(in: .whitespacesAndNewlines)
        // If email is provided, it must be valid
        if !trimmedEmail.isEmpty && !isValidEmail(trimmedEmail) {
            return false
        }
        // At least one field should have content (excluding empty strings)
        return !fullName.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty
            || !trimmedEmail.isEmpty
            || !description.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty
            || !website.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty
            || !location.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty
    }

    private func isValidEmail(_ email: String) -> Bool {
        let emailRegex = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,64}"
        let emailPredicate = NSPredicate(format: "SELF MATCHES %@", emailRegex)
        return emailPredicate.evaluate(with: email)
    }

    private func loadCurrentUserData() {
        guard let user = authManager.currentUser else { return }

        fullName = user.fullName ?? ""
        email = user.email ?? ""
        description = user.description ?? ""
        website = user.website ?? ""
        location = user.location ?? ""

        // Store original values
        originalFullName = fullName
        originalEmail = email
        originalDescription = description
        originalWebsite = website
        originalLocation = location
    }

    private func saveProfile() async {
        print("DEBUG: saveProfile called")
        guard isFormValid else {
            print("DEBUG: saveProfile - form is not valid")
            return
        }

        await MainActor.run {
            isLoading = true
            errorMessage = nil
        }

        do {
            print("DEBUG: saveProfile - preparing data")
            let trimmedFullName = fullName.trimmingCharacters(in: .whitespacesAndNewlines)
            let trimmedEmail = email.trimmingCharacters(in: .whitespacesAndNewlines)
            let trimmedDescription = description.trimmingCharacters(in: .whitespacesAndNewlines)
            let trimmedWebsite = website.trimmingCharacters(in: .whitespacesAndNewlines)
            let trimmedLocation = location.trimmingCharacters(in: .whitespacesAndNewlines)

            print("DEBUG: saveProfile - calling updateUserProfile")
            try await authManager.updateUserProfile(
                fullName: trimmedFullName.isEmpty ? nil : trimmedFullName,
                email: trimmedEmail.isEmpty ? nil : trimmedEmail,
                description: trimmedDescription.isEmpty ? nil : trimmedDescription,
                website: trimmedWebsite.isEmpty ? nil : trimmedWebsite,
                location: trimmedLocation.isEmpty ? nil : trimmedLocation
            )

            print("DEBUG: saveProfile - update successful, dismissing sheet")

            await MainActor.run {
                isLoading = false
                dismiss()
            }
        } catch {
            print("DEBUG: saveProfile - error occurred: \(error)")

            await MainActor.run {
                isLoading = false
                errorMessage = error.localizedDescription

                // Dismiss after showing error briefly
                DispatchQueue.main.asyncAfter(deadline: .now() + 2.0) {
                    dismiss()
                }
            }
        }
    }
}

#Preview {
    EditProfileView()
        .environmentObject(AuthenticationManager())
}
