//
//  ProfileView.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import SwiftUI

struct ProfileView: View {
    @EnvironmentObject var authManager: AuthenticationManager
    @EnvironmentObject var settingsManager: SettingsManager
    @State private var isLoading = false
    @State private var errorMessage: String?
    @State private var showingSettings = false
    @State private var showingLogoutConfirmation = false
    @State private var userRepositories: [Repository] = []
    @State private var userOrganizations: [Organization] = []
    @State private var showingEditProfile = false

    var body: some View {
        VStack(spacing: 0) {
            CustomHeader(
                title: "Profile",
                trailingContent: {
                    AnyView(
                        HStack(spacing: 12) {
                            Button(action: { showingEditProfile = true }) {
                                Image(systemName: "square.and.pencil")
                                    .font(.system(size: 18, weight: .medium))
                                    .foregroundColor(.primary)
                                    .frame(width: 44, height: 44)
                                    .modifier(GlassEffectModifier(shape: Circle()))
                            }
                            .buttonStyle(.borderless)

                            Button(action: { showingSettings = true }) {
                                Image(systemName: "gearshape")
                                    .font(.system(size: 18, weight: .medium))
                                    .foregroundColor(.primary)
                                    .frame(width: 44, height: 44)
                                    .modifier(GlassEffectModifier(shape: Circle()))
                            }
                            .buttonStyle(.borderless)
                        }
                    )
                }
            )

            ScrollView {
                VStack(spacing: 0) {
                    if isLoading {
                        LoadingView(message: "Loading profile...")
                    } else if let errorMessage = errorMessage {
                        ErrorView(message: errorMessage) {
                            Task { await refreshProfile() }
                        }
                    } else {
                        profileContent
                    }
                }
            }
        }
        .navigationBarHidden(true)
        .refreshable {
            await refreshProfile()
        }
        .sheet(isPresented: $showingSettings) {
            SettingsView()
        }
        .sheet(
            isPresented: $showingEditProfile,
            onDismiss: {
                Task {
                    await refreshProfile()
                }
            }
        ) {
            EditProfileView()
                .environmentObject(authManager)
                .environmentObject(settingsManager)
        }

        .confirmationDialog("Sign Out", isPresented: $showingLogoutConfirmation) {
            Button("Sign Out", role: .destructive) {
                authManager.logout()
            }
            Button("Cancel", role: .cancel) {}
        } message: {
            Text("Are you sure you want to sign out?")
        }
        .onAppear {
            if userRepositories.isEmpty && userOrganizations.isEmpty {
                Task { await loadUserData() }
            }

            // Navigation appearance is handled by SwiftUI
        }

    }

    private var profileContent: some View {
        VStack(spacing: 24) {
            profileHeader

            Divider()

            statisticsSection

            Divider()

            repositoriesSection

            if !userOrganizations.isEmpty {
                Divider()
                organizationsSection
            }

            Divider()

            accountSection
        }
        .padding()
    }

    private var profileHeader: some View {
        VStack(spacing: 16) {
            AsyncImage(url: URL(string: authManager.currentUser?.avatarUrl ?? "")) { phase in
                switch phase {
                case .success(let image):
                    image
                        .resizable()
                        .aspectRatio(contentMode: .fill)
                        .frame(width: 80, height: 80)
                        .clipped()
                case .failure(_):
                    Image(systemName: "person.circle.fill")
                        .font(.system(size: 80))
                        .foregroundColor(.gray)
                        .frame(width: 80, height: 80)
                case .empty:
                    Image(systemName: "person.circle.fill")
                        .font(.system(size: 80))
                        .foregroundColor(.gray)
                        .frame(width: 80, height: 80)
                @unknown default:
                    Image(systemName: "person.circle.fill")
                        .font(.system(size: 80))
                        .foregroundColor(.gray)
                        .frame(width: 80, height: 80)
                }
            }
            .frame(width: 80, height: 80)
            .clipShape(Circle())
            .overlay(
                Circle()
                    .stroke(Color.gray.opacity(0.3), lineWidth: 1)
            )

            VStack(spacing: 4) {
                if let fullName = authManager.currentUser?.fullName, !fullName.isEmpty {
                    Text(fullName)
                        .font(.title2)
                        .fontWeight(.semibold)
                } else {
                    Text(authManager.currentUser?.login ?? "Unknown")
                        .font(.title2)
                        .fontWeight(.semibold)
                }

                Text("@\(authManager.currentUser?.login ?? "unknown")")
                    .font(.subheadline)
                    .foregroundColor(.secondary)

                if let description = authManager.currentUser?.description, !description.isEmpty {
                    Text(description)
                        .font(.body)
                        .foregroundColor(.secondary)
                        .multilineTextAlignment(.center)
                        .padding(.top, 4)
                }

                if let location = authManager.currentUser?.location, !location.isEmpty {
                    HStack {
                        Image(systemName: "location")
                            .font(.caption)
                        Text(location)
                            .font(.caption)
                    }
                    .foregroundColor(.secondary)
                    .padding(.top, 2)
                }

                if let website = authManager.currentUser?.website, !website.isEmpty {
                    Link(destination: URL(string: website) ?? URL(string: "https://example.com")!) {
                        HStack {
                            Image(systemName: "link")
                                .font(.caption)
                            Text(website)
                                .font(.caption)
                        }
                        .foregroundColor(.accentColor)
                        .padding(.top, 2)
                    }
                }
            }
        }
    }

    private var statisticsSection: some View {
        VStack(alignment: .leading, spacing: 12) {
            Text("Statistics")
                .font(.headline)
                .frame(maxWidth: .infinity, alignment: .leading)

            HStack(spacing: 0) {
                StatisticView(
                    title: "Repositories",
                    value: userRepositories.count
                )

                Divider()
                    .frame(height: 40)

                StatisticView(
                    title: "Followers",
                    value: authManager.currentUser?.followersCount ?? 0
                )

                Divider()
                    .frame(height: 40)

                StatisticView(
                    title: "Following",
                    value: authManager.currentUser?.followingCount ?? 0
                )

                Divider()
                    .frame(height: 40)

                StatisticView(
                    title: "Stars",
                    value: authManager.currentUser?.starredReposCount ?? 0
                )
            }
            .padding(.vertical, 8)
        }
    }

    private var repositoriesSection: some View {
        VStack(alignment: .leading, spacing: 12) {
            HStack {
                Text("Recent Repositories")
                    .font(.headline)
                Spacer()
            }

            if userRepositories.isEmpty {
                Text("No repositories found")
                    .font(.subheadline)
                    .foregroundColor(.secondary)
                    .frame(maxWidth: .infinity, alignment: .center)
                    .padding()
            } else {
                VStack(spacing: 12) {
                    ForEach(Array(userRepositories.prefix(3))) { repository in
                        RepositoryCard(repository: repository)
                    }
                }
            }
        }
    }

    private var organizationsSection: some View {
        VStack(alignment: .leading, spacing: 12) {
            Text("Organizations")
                .font(.headline)
                .frame(maxWidth: .infinity, alignment: .leading)

            ScrollView(.horizontal, showsIndicators: false) {
                HStack(spacing: 12) {
                    ForEach(userOrganizations) { organization in
                        OrganizationCard(organization: organization)
                    }
                }
                .padding(.horizontal)
            }
        }
    }

    private var accountSection: some View {
        VStack(alignment: .leading, spacing: 12) {
            Text("Account")
                .font(.headline)
                .frame(maxWidth: .infinity, alignment: .leading)

            VStack(spacing: 8) {
                HStack {
                    Image(systemName: "server.rack")
                        .foregroundColor(.secondary)
                    Text("Server")
                        .font(.subheadline)
                    Spacer()
                    Text(
                        authManager.serverURL.replacingOccurrences(of: "https://", with: "")
                            .replacingOccurrences(of: "http://", with: "")
                    )
                    .font(.caption)
                    .foregroundColor(.secondary)
                }

                HStack {
                    Image(systemName: "person.badge.key")
                        .foregroundColor(.secondary)
                    Text("Account Type")
                        .font(.subheadline)
                    Spacer()
                    Text(authManager.currentUser?.isAdmin == true ? "Admin" : "User")
                        .font(.caption)
                        .foregroundColor(.secondary)
                }

                if let created = authManager.currentUser?.created {
                    HStack {
                        Image(systemName: "calendar")
                            .foregroundColor(.secondary)
                        Text("Member Since")
                            .font(.subheadline)
                        Spacer()
                        Text(created, style: .date)
                            .font(.caption)
                            .foregroundColor(.secondary)
                    }
                }
            }
            .padding(.vertical, 8)

            Button(action: { showingLogoutConfirmation = true }) {
                HStack {
                    Image(systemName: "rectangle.portrait.and.arrow.right")
                    Text("Sign Out")
                }
                .frame(maxWidth: .infinity)
                .padding()
                .background(Color.red)
                .foregroundColor(.white)
                .cornerRadius(10)
            }
            .padding(.top, 8)
        }
    }

    private func refreshProfile() async {
        errorMessage = nil
        await authManager.refreshUser()
        // await loadUserData()
    }

    func triggerRefresh() {
        Task {
            await refreshProfile()
        }
    }

    private func loadUserData() async {
        guard let apiService = authManager.getAPIService() else { return }

        isLoading = true
        errorMessage = nil

        defer {
            isLoading = false
        }

        do {
            async let repositories = apiService.getUserRepositories(limit: 10)
            async let organizations = apiService.getUserOrganizations()

            userRepositories = try await repositories
            userOrganizations = try await organizations
        } catch {
            // Handle cancellation and other errors gracefully
            if error is CancellationError {
                // Don't show error message for cancellation
                return
            }
            errorMessage = error.localizedDescription
        }
    }
}

struct StatisticView: View {
    let title: String
    let value: Int

    var body: some View {
        VStack(spacing: 4) {
            Text("\(value)")
                .font(.title2)
                .fontWeight(.semibold)
            Text(title)
                .font(.caption)
                .foregroundColor(.secondary)
        }
        .frame(maxWidth: .infinity)
    }
}

struct RepositoryCard: View {
    let repository: Repository

    var body: some View {
        VStack(alignment: .leading, spacing: 8) {
            HStack {
                Text(repository.name)
                    .font(.subheadline)
                    .fontWeight(.medium)
                    .lineLimit(1)

                if repository.private {
                    Image(systemName: "lock.fill")
                        .font(.caption)
                        .foregroundColor(.orange)
                }

                Spacer()

                HStack(spacing: 4) {
                    Image(systemName: "star")
                        .font(.caption)
                    Text("\(repository.stargazersCount)")
                        .font(.caption)
                }
                .foregroundColor(.secondary)
            }

            if let description = repository.description, !description.isEmpty {
                Text(description)
                    .font(.caption)
                    .foregroundColor(.secondary)
                    .lineLimit(2)
            }

            HStack {
                if let language = repository.language {
                    HStack(spacing: 4) {
                        Circle()
                            .fill(colorForLanguage(language))
                            .frame(width: 8, height: 8)
                        Text(language)
                            .font(.caption)
                            .foregroundColor(.secondary)
                    }
                }

                Spacer()

                Text("Updated \(repository.updatedAt.timeAgoSince())")
                    .font(.caption)
                    .foregroundColor(.secondary)
            }
        }
        .padding()
        .cornerRadius(8)
    }
}

struct OrganizationCard: View {
    let organization: Organization

    var body: some View {
        VStack(spacing: 8) {
            AsyncImage(url: URL(string: organization.avatarUrl)) { phase in
                switch phase {
                case .success(let image):
                    image
                        .resizable()
                        .aspectRatio(contentMode: .fill)
                        .frame(width: 50, height: 50)
                        .clipped()
                case .failure(_):
                    Image(systemName: "building.2")
                        .font(.title2)
                        .foregroundColor(.gray)
                        .frame(width: 50, height: 50)
                case .empty:
                    Image(systemName: "building.2")
                        .font(.title2)
                        .foregroundColor(.gray)
                        .frame(width: 50, height: 50)
                @unknown default:
                    Image(systemName: "building.2")
                        .font(.title2)
                        .foregroundColor(.gray)
                        .frame(width: 50, height: 50)
                }
            }
            .frame(width: 50, height: 50)
            .clipShape(Circle())

            Text(organization.username)
                .font(.caption)
                .fontWeight(.medium)
                .lineLimit(1)
        }
        .frame(width: 80)
    }
}

struct SettingsView: View {
    @EnvironmentObject var settingsManager: SettingsManager
    @EnvironmentObject var authManager: AuthenticationManager
    @Environment(\.dismiss) private var dismiss
    @State private var showingSignOutConfirmation = false

    var body: some View {
        NavigationView {
            Form {
                Section("Appearance") {
                    Picker("Theme", selection: $settingsManager.theme) {
                        ForEach(AppTheme.allCases) { theme in
                            Text(theme.displayName).tag(theme)
                        }
                    }

                    Picker("Accent Color", selection: $settingsManager.accentColor) {
                        ForEach(AppAccentColor.allCases) { color in
                            HStack {
                                Circle()
                                    .fill(color.color)
                                    .frame(width: 20, height: 20)
                                Text(color.displayName)
                            }
                            .tag(color)
                        }
                    }
                }

                Section("Repository Settings") {
                    Picker("List Style", selection: $settingsManager.repositoryListStyle) {
                        ForEach(RepositoryListStyle.allCases) { style in
                            Text(style.displayName).tag(style)
                        }
                    }

                    Picker("Default Sort", selection: $settingsManager.defaultRepositorySort) {
                        ForEach(RepositorySort.allCases) { sort in
                            Text(sort.displayName).tag(sort)
                        }
                    }

                    Toggle(
                        "Show Private Repositories", isOn: $settingsManager.showPrivateRepositories)
                    Toggle(
                        "Show Forked Repositories", isOn: $settingsManager.showForkedRepositories)
                }

                Section("Notifications") {
                    Toggle("Enable Notifications", isOn: $settingsManager.enableNotifications)
                    Toggle("Sound", isOn: $settingsManager.notificationSoundEnabled)
                }

                Section("Data") {
                    Toggle("Cache Images", isOn: $settingsManager.cacheImages)
                    Toggle("Auto Refresh", isOn: $settingsManager.enableAutoRefresh)

                    if settingsManager.enableAutoRefresh {
                        Picker("Refresh Interval", selection: $settingsManager.autoRefreshInterval)
                        {
                            ForEach(RefreshInterval.allCases) { interval in
                                Text(interval.displayName).tag(interval)
                            }
                        }
                    }
                }

                Section("Account") {
                    Button("Sign Out", role: .destructive) {
                        showingSignOutConfirmation = true
                    }
                }

                Section("About") {
                    HStack {
                        Text("Version")
                        Spacer()
                        Text("1.0.0")
                            .foregroundColor(.secondary)
                    }
                }
            }
            .navigationTitle("Settings")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Done") {
                        dismiss()
                    }
                }
            }
            .confirmationDialog("Sign Out", isPresented: $showingSignOutConfirmation) {
                Button("Sign Out", role: .destructive) {
                    dismiss()
                    authManager.logout()
                }
                Button("Cancel", role: .cancel) {}
            } message: {
                Text(
                    "Are you sure you want to sign out? You'll need to sign in again to access your repositories."
                )
            }
        }
    }
}

#Preview {
    MockProfileView()
        .environmentObject(PreviewAuthenticationManager())
        .environmentObject(PreviewSettingsManager())
}
