import Foundation
import SwiftUI

struct RepositoriesView: View {
    @EnvironmentObject private var authManager: AuthenticationManager
    @State private var showingCreateRepository = false
    @State private var repositories: [Repository] = []
    @State private var filteredRepositories: [Repository] = []
    @State private var isLoading = false
    @State private var errorMessage: String?
    @State private var searchText = ""
    @State private var selectedSortOption = SortOption.updated
    @State private var showingFilters = false
    @State private var showPrivateOnly = false
    @State private var showPublicOnly = false
    @Namespace private var glassNamespace
    @State private var isFetching = false

    enum SortOption: String, CaseIterable {
        case name = "Name"
        case updated = "Updated"
        case created = "Created"
        case stars = "Stars"
        case forks = "Forks"
    }

    var body: some View {
        NavigationStack {
            VStack(spacing: 0) {
                CustomHeader(
                    title: "Repositories",
                    trailingContent: {
                        AnyView(
                            HStack(spacing: 12) {
                                Button(action: { showingCreateRepository = true }) {
                                    Image(systemName: "plus")
                                        .font(.system(size: 18, weight: .medium))
                                        .foregroundColor(.primary)
                                        .frame(width: 44, height: 44)
                                        .modifier(GlassEffectModifier(shape: Circle()))
                                }
                                .buttonStyle(.borderless)

                                Button(action: {
                                    Task {
                                        await fetchRepositories()
                                    }
                                }) {
                                    Image(systemName: "arrow.clockwise")
                                        .font(.system(size: 18, weight: .medium))
                                        .foregroundColor(.primary)
                                        .frame(width: 44, height: 44)
                                        .modifier(GlassEffectModifier(shape: Circle()))
                                }
                                .buttonStyle(.borderless)

                                Button(action: { showingFilters = true }) {
                                    Image(systemName: "line.3.horizontal.decrease.circle")
                                        .font(.system(size: 18, weight: .medium))
                                        .foregroundColor(.primary)
                                        .frame(width: 44, height: 44)
                                        .modifier(GlassEffectModifier(shape: Circle()))
                                }
                                .buttonStyle(.borderless)
                            }
                        )
                    })

                if isLoading {
                    LoadingView(message: "Loading repositories...")
                } else if let errorMessage = errorMessage {
                    ErrorView(message: errorMessage) {
                        Task {
                            await fetchRepositories()
                        }
                    }
                } else if repositories.isEmpty {
                    EmptyStateView(
                        systemImage: "folder",
                        title: "No Repositories",
                        description:
                            "You don't have any repositories yet. Create your first repository to get started.",
                        actionTitle: "Create Repository"
                    ) {
                        showingCreateRepository = true
                    }
                } else {
                    List(filteredRepositories) { repository in
                        NavigationLink(
                            destination: RepositoryDetailView(
                                repository: repository,
                                onRepositoryUpdated: {
                                    // Don't automatically refresh - let user manually refresh if needed
                                    // This prevents infinite loops during navigation
                                })
                        ) {
                            CompactRepositoryRow(repository: repository)
                        }
                        .listRowInsets(EdgeInsets(top: 6, leading: 16, bottom: 6, trailing: 16))
                        .swipeActions(edge: .trailing, allowsFullSwipe: false) {
                            Button {
                                // Open in browser
                                if let url = URL(string: repository.htmlUrl) {
                                    UIApplication.shared.open(url)
                                }
                            } label: {
                                Image(systemName: "safari")
                                Text("Open")
                            }
                            .tint(.blue)

                            ShareLink(item: URL(string: repository.htmlUrl)!) {
                                Image(systemName: "square.and.arrow.up")
                                Text("Share")
                            }
                            .tint(.green)
                        }
                        .swipeActions(edge: .leading, allowsFullSwipe: false) {
                            Button {
                                // Copy clone URL
                                UIPasteboard.general.string = repository.cloneUrl
                            } label: {
                                Image(systemName: "doc.on.doc")
                                Text("Copy")
                            }
                            .tint(.orange)
                        }
                    }
                    .listStyle(.plain)
                    .searchable(text: $searchText, prompt: "Search repositories...")
                    .onChange(of: searchText) { _ in
                        applyFiltersAndSort()
                    }
                }
            }
            .sheet(
                isPresented: $showingCreateRepository,
                onDismiss: {
                    // Refresh repositories when create sheet is dismissed
                    DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                        Task {
                            await fetchRepositories()
                        }
                    }
                }
            ) {
                CreateRepositoryView()
                    .environmentObject(authManager)
            }
            .sheet(isPresented: $showingFilters) {
                FiltersView(
                    showPrivateOnly: $showPrivateOnly,
                    showPublicOnly: $showPublicOnly,
                    selectedSortOption: $selectedSortOption,
                    onApply: {
                        applyFiltersAndSort()
                    },
                    onRefresh: {
                        Task {
                            await fetchRepositories()
                        }
                    }
                )
            }
            .task {
                await fetchRepositories()
            }
        }
    }

    private func applyFiltersAndSort() {
        var filtered = repositories

        // Apply search filter
        if !searchText.isEmpty {
            filtered = filtered.filter { repository in
                repository.name.localizedCaseInsensitiveContains(searchText)
                    || repository.description?.localizedCaseInsensitiveContains(
                        searchText) == true
                    || repository.owner.login.localizedCaseInsensitiveContains(
                        searchText)
                    || repository.language?.localizedCaseInsensitiveContains(
                        searchText) == true
            }
        }

        // Apply visibility filters
        if showPrivateOnly {
            filtered = filtered.filter { $0.private }
        } else if showPublicOnly {
            filtered = filtered.filter { !$0.private }
        }

        // Apply sorting
        switch selectedSortOption {
        case .name:
            filtered = filtered.sorted {
                $0.name.localizedCaseInsensitiveCompare($1.name)
                    == .orderedAscending
            }
        case .updated:
            filtered = filtered.sorted { $0.updatedAt > $1.updatedAt }
        case .created:
            filtered = filtered.sorted { $0.createdAt > $1.createdAt }
        case .stars:
            filtered = filtered.sorted { $0.stargazersCount > $1.stargazersCount }
        case .forks:
            filtered = filtered.sorted { $0.forksCount > $1.forksCount }
        }

        filteredRepositories = filtered
    }

    private func fetchRepositories() async {
        // Prevent multiple simultaneous fetches
        if isFetching {
            return
        }

        guard let apiService = authManager.getAPIService() else {
            await MainActor.run {
                errorMessage = "Not authenticated"
            }
            return
        }

        await MainActor.run {
            isFetching = true
            isLoading = true
            errorMessage = nil
        }

        do {
            let fetchedRepositories = try await apiService.getUserRepositories()
            await MainActor.run {
                repositories = fetchedRepositories
                applyFiltersAndSort()
                isLoading = false
                isFetching = false
            }
        } catch {
            await MainActor.run {
                isLoading = false
                isFetching = false

                // Handle different types of cancellation errors gracefully - don't show error or trigger retry
                if error is CancellationError {
                    return
                }

                // Check for URL session cancellation
                if let nsError = error as NSError? {
                    if nsError.domain == NSURLErrorDomain && nsError.code == NSURLErrorCancelled {
                        return
                    }
                }

                // Check for string-based cancellation
                if error.localizedDescription.lowercased().contains("cancelled") {
                    return
                }

                // Only show error for non-cancellation errors
                errorMessage = error.localizedDescription
            }
        }
    }
}

struct FiltersView: View {
    @Binding var showPrivateOnly: Bool
    @Binding var showPublicOnly: Bool
    @Binding var selectedSortOption: RepositoriesView.SortOption
    let onApply: () -> Void
    let onRefresh: () -> Void
    @Environment(\.dismiss) private var dismiss

    var body: some View {
        NavigationView {
            Form {
                Section("Visibility") {
                    Toggle("Private repositories only", isOn: $showPrivateOnly)
                        .onChange(of: showPrivateOnly) { newValue in
                            if newValue {
                                showPublicOnly = false
                            }
                        }

                    Toggle("Public repositories only", isOn: $showPublicOnly)
                        .onChange(of: showPublicOnly) { newValue in
                            if newValue {
                                showPrivateOnly = false
                            }
                        }
                }

                Section("Sort By") {
                    Picker("Sort Option", selection: $selectedSortOption) {
                        ForEach(RepositoriesView.SortOption.allCases, id: \.self) { option in
                            Text(option.rawValue).tag(option)
                        }
                    }
                    .pickerStyle(.wheel)
                }

                Section {
                    Button("Clear All Filters") {
                        showPrivateOnly = false
                        showPublicOnly = false
                        selectedSortOption = .updated
                        onApply()
                    }

                    Button("Refresh") {
                        onRefresh()
                        dismiss()
                    }
                }
            }
            .navigationTitle("Filters & Sort")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button("Cancel") {
                        dismiss()
                    }
                }

                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Apply") {
                        onApply()
                        dismiss()
                    }
                }
            }
        }
    }
}

struct CompactRepositoryRow: View {
    let repository: Repository

    var body: some View {
        HStack(spacing: 12) {
            VStack(alignment: .leading, spacing: 2) {
                HStack(spacing: 6) {
                    Text(repository.name)
                        .font(.system(size: 16, weight: .medium))
                        .foregroundColor(.primary)
                        .lineLimit(1)

                    if repository.private {
                        Image(systemName: "lock.fill")
                            .font(.system(size: 12))
                            .foregroundColor(.secondary)
                    }

                    Spacer()
                }

                if let description = repository.description, !description.isEmpty {
                    Text(description)
                        .font(.system(size: 13))
                        .foregroundColor(.secondary)
                        .lineLimit(1)
                }

                HStack(spacing: 12) {
                    if let language = repository.language {
                        HStack(spacing: 4) {
                            Circle()
                                .fill(colorForLanguage(language))
                                .frame(width: 8, height: 8)
                            Text(language)
                                .font(.system(size: 12))
                                .foregroundColor(.secondary)
                        }
                    }

                    if repository.stargazersCount > 0 {
                        HStack(spacing: 2) {
                            Image(systemName: "star.fill")
                                .font(.system(size: 10))
                            Text("\(repository.stargazersCount)")
                                .font(.system(size: 12))
                        }
                        .foregroundColor(.secondary)
                    }

                    if repository.forksCount > 0 {
                        HStack(spacing: 2) {
                            Image(systemName: "tuningfork")
                                .font(.system(size: 10))
                            Text("\(repository.forksCount)")
                                .font(.system(size: 12))
                        }
                        .foregroundColor(.secondary)
                    }

                    Spacer()

                    Text(repository.updatedAt.timeAgoSince())
                        .font(.system(size: 11))
                        .foregroundColor(.secondary)
                }
            }

        }
        .padding(.vertical, 4)
        .contentShape(Rectangle())
    }
}
