//
//  RepositoryDetailView.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import SwiftUI

struct RepositoryDetailView: View {
    @State private var repository: Repository
    let onRepositoryUpdated: (() -> Void)?
    @Environment(\.dismiss) private var dismiss

    init(
        repository: Repository, onRepositoryUpdated: (() -> Void)? = nil
    ) {
        self._repository = State(initialValue: repository)
        self.onRepositoryUpdated = onRepositoryUpdated
    }

    @EnvironmentObject var authManager: AuthenticationManager
    @EnvironmentObject var settingsManager: SettingsManager
    @State private var isLoading = false
    @State private var errorMessage: String?
    @State private var selectedTab = 0
    @State private var branches: [Branch] = []
    @State private var releases: [Release] = []
    @State private var issues: [Issue] = []
    @State private var isStarred = false
    @State private var isWatching = false
    @State private var showingBranches = false
    @State private var showingRepositorySettings = false
    @State private var repositoryDeleted = false

    var body: some View {
        ZStack {
            VStack(spacing: 0) {
                CustomHeader(
                    title: repository.name,
                    showBackButton: false,
                    trailingContent: {
                        AnyView(
                            Menu {
                                Button(action: { toggleStar() }) {
                                    HStack {
                                        Image(systemName: isStarred ? "star.fill" : "star")
                                        Text(isStarred ? "Unstar" : "Star")
                                    }
                                }

                                Button(action: { toggleWatch() }) {
                                    HStack {
                                        Image(systemName: isWatching ? "eye.fill" : "eye")
                                        Text(isWatching ? "Unwatch" : "Watch")
                                    }
                                }

                                Divider()

                                Button(action: { showingBranches = true }) {
                                    HStack {
                                        Image(systemName: "arrow.triangle.branch")
                                        Text("Branches")
                                    }
                                }

                                Divider()

                                Button(action: { showingRepositorySettings = true }) {
                                    HStack {
                                        Image(systemName: "gearshape")
                                        Text("Settings")
                                    }
                                }
                            } label: {
                                Image(systemName: "ellipsis.circle")
                                    .font(.system(size: 18, weight: .medium))
                                    .foregroundColor(.primary)
                                    .frame(width: 44, height: 44)
                                    .modifier(GlassEffectModifier(shape: Circle()))
                            }
                            .menuStyle(.borderlessButton)
                        )
                    }
                )

                ScrollView {
                    VStack(spacing: 0) {
                        repositoryHeader

                        Divider()

                        tabSelector

                        Divider()

                        tabContent
                    }
                }
            }
        }
        .sheet(isPresented: $showingBranches) {
            BranchesSheet(branches: branches, repository: repository)
        }
        .sheet(isPresented: $showingRepositorySettings) {
            RepositorySettingsView(
                repository: repository,
                onRepositoryUpdated: { updatedRepository in
                    repository = updatedRepository
                    onRepositoryUpdated?()
                },
                onRepositoryDeleted: {
                    repositoryDeleted = true
                    dismiss()
                    onRepositoryUpdated?()
                }
            )
            .environmentObject(authManager)
        }
        .onAppear {
            Task { await loadRepositoryData() }
        }
    }

    private var repositoryHeader: some View {
        VStack(alignment: .leading, spacing: 16) {
            HStack {
                VStack(alignment: .leading, spacing: 4) {
                    HStack {
                        Text(repository.owner.login)
                            .font(.system(size: 16, weight: .medium))
                            .foregroundColor(.primary)

                        if repository.private {
                            Image(systemName: "lock.fill")
                                .font(.system(size: 14))
                                .foregroundColor(.orange)
                        }

                        if repository.fork {
                            Image(systemName: "tuningfork")
                                .font(.system(size: 14))
                                .foregroundColor(.secondary)
                        }

                        Spacer()
                    }
                }

                Spacer()

                AsyncImage(url: URL(string: repository.owner.avatarUrl)) { phase in
                    switch phase {
                    case .success(let image):
                        image
                            .resizable()
                            .aspectRatio(contentMode: .fill)
                    case .failure(_):
                        Image(systemName: "person.circle.fill")
                            .foregroundColor(.secondary)
                    case .empty:
                        ProgressView()
                            .progressViewStyle(CircularProgressViewStyle(tint: .secondary))
                    @unknown default:
                        Image(systemName: "person.circle.fill")
                            .foregroundColor(.secondary)
                    }
                }
                .frame(width: 40, height: 40)
                .clipShape(Circle())
            }

            if let description = repository.description, !description.isEmpty {
                Text(description)
                    .font(.system(size: 15))
                    .foregroundColor(.secondary)
                    .lineLimit(3)
            }

            HStack(spacing: 20) {
                StatItem(icon: "star", count: repository.stargazersCount, label: "Stars")
                StatItem(icon: "tuningfork", count: repository.forksCount, label: "Forks")
                StatItem(
                    icon: "exclamationmark.circle", count: repository.openIssuesCount,
                    label: "Issues")
                StatItem(icon: "arrow.triangle.pull", count: repository.openPrCounter, label: "PRs")

                Spacer()
            }

            HStack {
                if let language = repository.language {
                    HStack(spacing: 4) {
                        Circle()
                            .fill(colorForLanguage(language))
                            .frame(width: 12, height: 12)
                        Text(language)
                            .font(.system(size: 12))
                            .foregroundColor(.secondary)
                    }
                }

                Spacer()

                Text("Updated \(repository.updatedAt.timeAgoSince())")
                    .font(.system(size: 12))
                    .foregroundColor(.secondary)
            }

            if !repository.topics.isEmpty {
                ScrollView(.horizontal, showsIndicators: false) {
                    HStack(spacing: 8) {
                        ForEach(repository.topics, id: \.self) { topic in
                            Text(topic)
                                .font(.system(size: 11))
                                .padding(.horizontal, 8)
                                .padding(.vertical, 4)
                                .background(Color.blue.opacity(0.1))
                                .foregroundColor(.blue)
                                .cornerRadius(12)
                        }
                    }
                    .padding(.horizontal, 16)
                }
            }
        }
        .padding(16)
    }

    private var tabSelector: some View {
        ScrollView(.horizontal, showsIndicators: false) {
            HStack(spacing: 0) {
                TabButton(title: "Issues (\(issues.count))", isSelected: selectedTab == 0) {
                    selectedTab = 0
                }
                TabButton(title: "Releases (\(releases.count))", isSelected: selectedTab == 1) {
                    selectedTab = 1
                }
            }
            .padding(.horizontal, 16)
        }
        .padding(.vertical, 4)
    }

    private var tabContent: some View {
        Group {
            switch selectedTab {
            case 0:
                issuesTab
            case 1:
                releasesTab
            default:
                EmptyView()
            }
        }
        .padding()
    }

    private var issuesTab: some View {
        VStack(alignment: .leading, spacing: 16) {
            HStack {
                let openIssuesCount = issues.filter { $0.state == .open }.count
                let closedIssuesCount = issues.filter { $0.state == .closed }.count

                Text("Issues (\(issues.count))")
                    .font(.headline)

                Spacer()

                HStack(spacing: 12) {
                    if openIssuesCount > 0 {
                        SwiftUI.Label(
                            "\(openIssuesCount) Open", systemImage: "exclamationmark.circle"
                        )
                        .font(.caption)
                        .foregroundColor(.green)
                    }

                    if closedIssuesCount > 0 {
                        SwiftUI.Label(
                            "\(closedIssuesCount) Closed", systemImage: "checkmark.circle"
                        )
                        .font(.caption)
                        .foregroundColor(.secondary)
                    }
                }
            }

            if isLoading {
                ProgressView("Loading issues...")
                    .frame(maxWidth: .infinity, maxHeight: .infinity)
            } else if let errorMessage = errorMessage {
                VStack(spacing: 20) {
                    Image(systemName: "exclamationmark.triangle")
                        .font(.system(size: 50))
                        .foregroundColor(.orange)

                    Text("Error Loading Issues")
                        .font(.title2)
                        .fontWeight(.semibold)

                    Text(errorMessage)
                        .font(.subheadline)
                        .foregroundColor(.secondary)
                        .multilineTextAlignment(.center)

                    Button("Retry") {
                        Task {
                            await loadRepositoryData()
                        }
                    }
                    .buttonStyle(.borderedProminent)
                }
                .frame(maxWidth: .infinity, maxHeight: .infinity)
                .padding()
            } else if issues.isEmpty {
                VStack(spacing: 20) {
                    Image(systemName: "exclamationmark.circle")
                        .font(.system(size: 50))
                        .foregroundColor(.secondary)

                    Text("No Issues")
                        .font(.title2)
                        .fontWeight(.semibold)

                    Text("Issues help you track bugs, feature requests, and other project tasks.")
                        .font(.subheadline)
                        .foregroundColor(.secondary)
                        .multilineTextAlignment(.center)
                }
                .frame(maxWidth: .infinity, maxHeight: .infinity)
                .padding()
            } else {
                ScrollView {
                    LazyVStack(spacing: 12) {
                        ForEach(issues) { issue in
                            IssueRowView(issue: issue)
                        }
                    }
                }
            }
        }
    }

    private var releasesTab: some View {
        VStack(alignment: .leading, spacing: 16) {
            HStack {
                Text("Releases")
                    .font(.headline)

                Spacer()
            }

            if releases.isEmpty {
                VStack(spacing: 20) {
                    Image(systemName: "tag")
                        .font(.system(size: 50))
                        .foregroundColor(.secondary)

                    Text("No Releases")
                        .font(.title2)
                        .fontWeight(.semibold)

                    Text("Releases are snapshots of your project at specific points in time.")
                        .font(.subheadline)
                        .foregroundColor(.secondary)
                        .multilineTextAlignment(.center)
                }
                .frame(maxWidth: .infinity, maxHeight: .infinity)
                .padding()
            } else {
                ScrollView {
                    LazyVStack(spacing: 12) {
                        ForEach(releases) { release in
                            ReleaseRowView(release: release)
                        }
                    }
                }
            }
        }
    }

    func loadRepositoryData() async {
        guard let apiService = authManager.getAPIService() else { return }

        isLoading = true
        errorMessage = nil

        do {
            async let branchesTask = apiService.getRepositoryBranches(
                owner: repository.owner.login, repo: repository.name)
            async let releasesTask = apiService.getRepositoryReleases(
                owner: repository.owner.login, repo: repository.name, limit: 10)
            branches = try await branchesTask
            releases = try await releasesTask

            // Load issues using the 'all' state to get both open and closed issues
            issues = try await apiService.getRepositoryIssues(
                owner: repository.owner.login, repo: repository.name, state: .all, limit: 30)
        } catch {
            errorMessage = error.localizedDescription
        }

        isLoading = false
    }

    func toggleStar() {
        guard let apiService = authManager.getAPIService() else { return }

        Task {
            do {
                if isStarred {
                    try await apiService.unstarRepository(
                        owner: repository.owner.login, repo: repository.name)
                    repository = createUpdatedRepository(starsDelta: -1)
                } else {
                    try await apiService.starRepository(
                        owner: repository.owner.login, repo: repository.name)
                    repository = createUpdatedRepository(starsDelta: 1)
                }
                isStarred.toggle()
                onRepositoryUpdated?()
            } catch {
                errorMessage = error.localizedDescription
            }
        }
    }

    func toggleWatch() {
        guard let apiService = authManager.getAPIService() else { return }

        Task {
            do {
                if isWatching {
                    try await apiService.unwatchRepository(
                        owner: repository.owner.login, repo: repository.name)
                    repository = createUpdatedRepository(watchersDelta: -1)
                } else {
                    try await apiService.watchRepository(
                        owner: repository.owner.login, repo: repository.name)
                    repository = createUpdatedRepository(watchersDelta: 1)
                }
                isWatching.toggle()
                onRepositoryUpdated?()
            } catch {
                errorMessage = error.localizedDescription
            }
        }
    }

    func createUpdatedRepository(starsDelta: Int = 0, watchersDelta: Int = 0) -> Repository {
        return Repository(
            id: repository.id,
            name: repository.name,
            fullName: repository.fullName,
            description: repository.description,
            htmlUrl: repository.htmlUrl,
            cloneUrl: repository.cloneUrl,
            sshUrl: repository.sshUrl,
            owner: repository.owner,
            private: repository.private,
            fork: repository.fork,
            template: repository.template,
            empty: repository.empty,
            archived: repository.archived,
            mirror: repository.mirror,
            size: repository.size,
            language: repository.language,
            languagesUrl: repository.languagesUrl,
            forksCount: repository.forksCount,
            stargazersCount: repository.stargazersCount + starsDelta,
            watchersCount: repository.watchersCount + watchersDelta,
            openIssuesCount: repository.openIssuesCount,
            openPrCounter: repository.openPrCounter,
            releaseCounter: repository.releaseCounter,
            defaultBranch: repository.defaultBranch,
            createdAt: repository.createdAt,
            updatedAt: repository.updatedAt,
            permissions: repository.permissions,
            hasIssues: repository.hasIssues,
            hasWiki: repository.hasWiki,
            hasPullRequests: repository.hasPullRequests,
            hasProjects: repository.hasProjects,
            hasReleases: repository.hasReleases,
            hasPackages: repository.hasPackages,
            hasActions: repository.hasActions,
            topics: repository.topics,
            avatarUrl: repository.avatarUrl,
            internalTracker: repository.internalTracker,
            externalTracker: repository.externalTracker,
            externalWiki: repository.externalWiki
        )
    }
}

struct StatItem: View {
    let icon: String
    let count: Int
    let label: String

    var body: some View {
        VStack(alignment: .leading, spacing: 2) {
            HStack(spacing: 4) {
                Image(systemName: icon)
                    .font(.system(size: 12))
                    .foregroundColor(.secondary)
                Text("\(count)")
                    .font(.system(size: 14, weight: .medium))
                    .foregroundColor(.primary)
            }

            Text(label)
                .font(.system(size: 10))
                .foregroundColor(.secondary)
        }
    }
}

struct TabButton: View {
    let title: String
    let isSelected: Bool
    let action: () -> Void

    var body: some View {
        Button(action: action) {
            Text(title)
                .font(.system(size: 14, weight: isSelected ? .semibold : .regular))
                .foregroundColor(isSelected ? .primary : .secondary)
                .padding(.horizontal, 16)
                .padding(.vertical, 8)
                .background(
                    RoundedRectangle(cornerRadius: 8)
                        .fill(isSelected ? Color.accentColor.opacity(0.1) : Color.clear)
                )
                .overlay(
                    RoundedRectangle(cornerRadius: 8)
                        .stroke(isSelected ? Color.accentColor : Color.clear, lineWidth: 1)
                )
        }
        .buttonStyle(PlainButtonStyle())
    }
}

struct CommitRowView: View {
    let commit: Commit

    var body: some View {
        HStack {
            VStack(alignment: .leading, spacing: 2) {
                Text(commit.commit.message.components(separatedBy: "\n").first ?? "")
                    .font(.system(size: 14, weight: .medium))
                    .lineLimit(1)

                HStack {
                    if let author = commit.author {
                        Text(author.login)
                            .font(.system(size: 12))
                            .foregroundColor(.secondary)
                    }

                    Text("•")
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)

                    Text(commit.created.timeAgoSince())
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)
                }
            }

            Spacer()

            Text(String(commit.sha.prefix(7)))
                .font(.system(size: 11, design: .monospaced))
                .padding(.horizontal, 6)
                .padding(.vertical, 2)
                .background(Color.secondary.opacity(0.1))
                .cornerRadius(4)
        }
        .padding(.vertical, 2)
    }
}

struct IssueRowView: View {
    let issue: Issue

    var body: some View {
        HStack {
            Image(
                systemName: issue.state == .open ? "exclamationmark.circle" : "checkmark.circle"
            )
            .foregroundColor(issue.state == .open ? .green : .purple)
            .font(.system(size: 12))

            VStack(alignment: .leading, spacing: 2) {
                Text(issue.title)
                    .font(.system(size: 14, weight: .medium))
                    .lineLimit(1)

                HStack {
                    Text("#\(issue.number)")
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)

                    Text("•")
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)

                    Text(issue.createdAt.timeAgoSince())
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)
                }
            }

            Spacer()
        }
        .padding(.vertical, 2)
    }
}

struct ReleaseRowView: View {
    let release: Release

    var body: some View {
        HStack {
            Image(systemName: "tag")
                .foregroundColor(.blue)
                .font(.system(size: 12))

            VStack(alignment: .leading, spacing: 2) {
                Text(release.name)
                    .font(.system(size: 14, weight: .medium))
                    .lineLimit(1)

                HStack {
                    Text(release.tagName)
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)

                    Text("•")
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)

                    Text(release.createdAt.timeAgoSince())
                        .font(.system(size: 12))
                        .foregroundColor(.secondary)

                    if release.prerelease {
                        Text("Pre-release")
                            .font(.system(size: 10))
                            .padding(.horizontal, 4)
                            .padding(.vertical, 1)
                            .background(Color.orange.opacity(0.2))
                            .foregroundColor(.orange)
                            .cornerRadius(4)
                    }
                }
            }

            Spacer()
        }
        .padding(.vertical, 2)
    }
}

struct BranchesSheet: View {
    let branches: [Branch]
    let repository: Repository
    @Environment(\.dismiss) private var dismiss

    var body: some View {
        NavigationView {
            List(branches, id: \.name) { branch in
                HStack {
                    Image(systemName: "arrow.triangle.branch")
                        .foregroundColor(.secondary)
                        .font(.caption)

                    VStack(alignment: .leading, spacing: 4) {
                        Text(branch.name)
                            .font(.subheadline)
                            .fontWeight(
                                branch.name == repository.defaultBranch ? .semibold : .regular)

                        if branch.name == repository.defaultBranch {
                            Text("Default branch")
                                .font(.caption)
                                .foregroundColor(.secondary)
                        }
                    }

                    Spacer()

                    if branch.protected {
                        Image(systemName: "lock.fill")
                            .foregroundColor(.orange)
                            .font(.caption)
                    }
                }
                .padding(.vertical, 4)
            }
            .navigationTitle("Branches")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Done") {
                        dismiss()
                    }
                }
            }
        }
    }
}

struct RepositorySettingsView: View {
    let repository: Repository
    let onRepositoryUpdated: ((Repository) -> Void)?
    let onRepositoryDeleted: (() -> Void)?
    @EnvironmentObject var authManager: AuthenticationManager
    @Environment(\.dismiss) private var dismiss

    @State private var name: String
    @State private var description: String
    @State private var website: String
    @State private var isPrivate: Bool
    @State private var hasIssues: Bool
    @State private var hasWiki: Bool
    @State private var hasPullRequests: Bool
    @State private var defaultBranch: String
    @State private var topics: String
    @State private var isSaving = false
    @State private var errorMessage: String?
    @State private var showingDeleteConfirmation = false

    init(
        repository: Repository, onRepositoryUpdated: ((Repository) -> Void)? = nil,
        onRepositoryDeleted: (() -> Void)? = nil
    ) {
        self.repository = repository
        self.onRepositoryUpdated = onRepositoryUpdated
        self.onRepositoryDeleted = onRepositoryDeleted
        self._name = State(initialValue: repository.name)
        self._description = State(initialValue: repository.description ?? "")
        self._website = State(initialValue: "")
        self._isPrivate = State(initialValue: repository.private)
        self._hasIssues = State(initialValue: repository.hasIssues)
        self._hasWiki = State(initialValue: repository.hasWiki)
        self._hasPullRequests = State(initialValue: repository.hasPullRequests)
        self._defaultBranch = State(initialValue: repository.defaultBranch)
        self._topics = State(initialValue: repository.topics.joined(separator: ", "))
    }

    var body: some View {
        NavigationView {
            Form {
                Section("Repository Details") {
                    TextField("Repository name", text: $name)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)

                    TextField("Description", text: $description, axis: .vertical)
                        .lineLimit(3...6)

                    TextField("Website", text: $website)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)
                        .keyboardType(.URL)

                    TextField("Topics (comma separated)", text: $topics)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)
                }

                Section("Visibility") {
                    Picker("Visibility", selection: $isPrivate) {
                        Text("Public").tag(false)
                        Text("Private").tag(true)
                    }
                    .pickerStyle(SegmentedPickerStyle())
                }

                Section("Features") {
                    Toggle("Issues", isOn: $hasIssues)
                    Toggle("Wiki", isOn: $hasWiki)
                    Toggle("Pull Requests", isOn: $hasPullRequests)
                }

                Section("Repository") {
                    TextField("Default branch", text: $defaultBranch)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)
                }

                Section("Danger Zone") {
                    Button("Delete Repository", role: .destructive) {
                        showingDeleteConfirmation = true
                    }
                    .disabled(isSaving)
                }

                if let errorMessage = errorMessage {
                    Section {
                        Text(errorMessage)
                            .foregroundColor(.red)
                            .font(.caption)
                    }
                }
            }
            .navigationTitle("Repository Settings")
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button("Cancel") {
                        dismiss()
                    }
                }

                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Save") {
                        Task { await saveSettings() }
                    }
                    .disabled(isSaving || name.isEmpty)
                }
            }
            .confirmationDialog("Delete Repository", isPresented: $showingDeleteConfirmation) {
                Button("Delete Repository", role: .destructive) {
                    Task { await deleteRepository() }
                }
                Button("Cancel", role: .cancel) {}
            } message: {
                Text(
                    "Are you sure you want to delete this repository? This action cannot be undone."
                )
            }
        }
    }

    private func saveSettings() async {
        isSaving = true
        errorMessage = nil

        guard let apiService = authManager.getAPIService() else {
            errorMessage = "Authentication error"
            isSaving = false
            return
        }

        do {
            let updateData: [String: Any] = [
                "name": name,
                "description": description,
                "website": website.isEmpty ? nil : website,
                "private": isPrivate,
                "has_issues": hasIssues,
                "has_wiki": hasWiki,
                "has_pull_requests": hasPullRequests,
                "default_branch": defaultBranch,
                "topics": topics.split(separator: ",").map {
                    $0.trimmingCharacters(in: .whitespacesAndNewlines)
                }.filter { !$0.isEmpty },
            ].compactMapValues { $0 }

            let updatedRepository = try await apiService.updateRepository(
                owner: repository.owner.login,
                repo: repository.name,
                updateData: updateData
            )

            await MainActor.run {
                isSaving = false
                onRepositoryUpdated?(updatedRepository)
                // Add small delay to prevent race condition between callback and dismiss
                DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                    dismiss()
                }
            }
        } catch {
            await MainActor.run {
                errorMessage = error.localizedDescription
                isSaving = false
            }
        }
    }

    private func deleteRepository() async {
        isSaving = true
        errorMessage = nil

        guard let apiService = authManager.getAPIService() else {
            errorMessage = "Authentication error"
            isSaving = false
            return
        }

        do {
            try await apiService.deleteRepository(
                owner: repository.owner.login,
                repo: repository.name
            )
            await MainActor.run {
                isSaving = false
                dismiss()
                // Add small delay to prevent race condition between dismiss and callback
                DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                    onRepositoryDeleted?()
                }
            }
        } catch {
            await MainActor.run {
                errorMessage = error.localizedDescription
                isSaving = false
            }
        }
    }
}
